﻿#ifndef NOUN_H_INCLUDED
#define NOUN_H_INCLUDED

#include <string>
#include <map>
#include <set>
#include <vector>
#include <fstream>

#include "boost/algorithm/string.hpp"

enum NounGender
{
	NG_MALE = 0,
	NG_FEMALE,
	NG_NEUTRAL
};

struct NounRecord
{
	std::wstring nominativeForm;
	NounGender gender;

	bool haveSingleForm;
	bool haveMultipleForm;

	bool haveStandardMultipleForm;
	bool haveStandardMultipleFormWithMissingLastVowel;
	
	std::wstring specialMultipleForm;

	bool canBeAnimate;
	bool canBeInanimate;

	NounRecord();
	NounRecord(std::wstring line);
};

enum NounDeclencion
{
	FIRST_A_IFORM_SINGULAR = 0,
	FIRST_A_IFORM_INANIMATE_PLURAL,
	FIRST_A_IFORM_ANIMATE_PLURAL,
	FIRST_A_UFORM_SINGULAR,
	FIRST_A_UFORM_INANIMATE_PLURAL,
	FIRST_A_UFORM_ANIMATE_PLURAL,
	FIRST_YA_FORM_SINGULAR,
	FIRST_YA_FORM_INANIMATE_PLURAL,
	FIRST_YA_FORM_ANIMATE_PLURAL,
	SECOND_MALE_IFORM_INANIMATE_SINGULAR,
	SECOND_MALE_IFORM_INANIMATE_PLURAL,
	SECOND_MALE_IFORM_ANIMATE_SINGULAR,
	SECOND_MALE_IFORM_ANIMATE_PLURAL,
	SECOND_MALE_UFORM_INANIMATE_SINGULAR,
	SECOND_MALE_UFORM_INANIMATE_PLURAL,
	SECOND_MALE_UFORM_ANIMATE_SINGULAR,
	SECOND_MALE_UFORM_ANIMATE_PLURAL,
	SECOND_MALE_SSFORM_INANIMATE_SINGULAR,
	SECOND_MALE_SSFORM_INANIMATE_PLURAL,
	SECOND_MALE_SSFORM_ANIMATE_SINGULAR,
	SECOND_MALE_SSFORM_ANIMATE_PLURAL,
	SECOND_NEUTRAL_E_FORM_SINGULAR,
	SECOND_NEUTRAL_E_FORM_PLURAL,
	SECOND_NEUTRAL_O_FORM_SINGULAR,
	SECOND_NEUTRAL_O_FORM_PLURAL,
	THIRD_FORM_SINGULAR,
	THIRD_FORM_INANIMATE_PLURAL,
	THIRD_FORM_ANIMATE_PLURAL
};

enum NounGrammaticalCase
{
	NGC_P1_NOMINATIVE = 0,
	NGC_P2_GENITIVE,
	NGC_P3_DATIVE,
	NGC_P4_ACCUSATIVE,
	NGC_P5_INSTRUMENTAL,
	NGC_P6_PREPOSITIONAL
};

struct NounDeclencionCaseTableRecord
{
	NounDeclencion nounDeclencion;
	std::vector<std::pair<NounGrammaticalCase, std::wstring>> grammaticalCaseMap;

};

extern std::vector<NounRecord> NounRecordArr;

extern std::vector<NounDeclencionCaseTableRecord> nounDeclencionCaseTable;


NounDeclencion WStringToNounDeclencion(std::wstring str);
std::wstring NounGrammaticalCaseToWString(NounGrammaticalCase nounGrammaticalCase);
NounGrammaticalCase WStringToNounGrammaticalCase(std::wstring str);

//std::wstring NounNumberToWString(NounNumber nounNumber);

typedef std::tuple<NounDeclencion, NounGrammaticalCase> NounTuple;

typedef std::set<std::wstring> StringSet;

std::vector<std::wstring> GetAllNounEndingArr();

bool NounIsInDictionary(std::wstring nounNominative);
bool NounSpecialPluralFormIsInDictionary(std::wstring nounNominativePlural);


NounRecord GetNounRecordFromDictionary(std::wstring nounNominative);
NounRecord GetNounRecordFromDictionary_BySpecialPluralForm(std::wstring nounNominativePlural);

bool charIsConsolant(wchar_t c);

bool charIsVowel(wchar_t c);


struct NounStruct
{
	NounGrammaticalCase nounGrammaticalCase;
	NounRecord nounRecord;
};


std::set<std::pair<std::wstring, std::wstring>> getPossibleNounEndingDivisionSet(std::wstring noun);

std::vector<NounTuple> GetPossibleNounTupleArr(std::wstring nounEnding);

bool NounDeclencionIsPlural(NounDeclencion nounDeclencion);

NounDeclencion ConvertToSingularIfNeeded(NounDeclencion nounDeclencion);

std::wstring GetNounNoninativeSingular(std::wstring nounBase, NounDeclencion nounDeclencion);

std::wstring GetNounNoninativePlural(std::wstring nounBase, NounDeclencion nounDeclencion);

std::wstring GetNounNoninative(std::wstring nounBase, NounDeclencion nounDeclencion);

void SetupDeclentionMap();

bool NounFitsDeclention(NounRecord nounRecord, NounTuple nounTuple);

std::vector<NounStruct> RecognizeNoun(std::wstring noun);

NounDeclencion CalculateNounDeclention(NounRecord nounRecord);


void LoadFrequentWordSet();
void LoadNounDeclencionCaseTable();

#endif //NOUN_H_INCLUDED
